<?php

namespace Drupal\gtfs_schedule\Form;

use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;

class GTFSScheduleDebugForm extends FormBase {
  public function getFormId() {
    return 'gtfs_schedule_debug_form';
  }

  public function buildForm(array $form, FormStateInterface $form_state) {
    $directory = GTFS_SCHEDULE_CACHE_LOCATION;
    $form['actions']['#type'] = 'actions';
    $files = gtfs_schedule_get_cache_files();
    $stale_files = gtfs_schedule_get_cache_files(true);
    $form['cache_action'] = [
      '#type' => 'radios',
      '#options' => [
        '' => $this->t('Leave cache alone'),
        'clear_cache' => $this->t('Delete GTFS schedule request cache (count: :count)', [':count' => (count($files))]),
        'invalidate_cache' => $this->t('Invalidate GTFS schedule request cache (count: :count)', [':count' => (count($files))]),
        'clear_stale_cache' => $this->t('Delete stale GTFS requests cache. (count: :count)', [':count' => (count($stale_files))]),
        'clear_url' => $this->t('Clear single URL (see below)')
      ],
      '#default_value' => '',
      'invalidate_cache' => ['#description' => $this->t('Invalidated cache files will become stale.')]
    ];

    $form['clear_url_cache'] = [
      '#type' => 'textfield',
      '#name' => 'clear_url_cache',
      '#title' => 'Clear cache for URL',
      '#maxlength' => 512,
    ];

    $form['actions']['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Submit'),
    ];

    return $form;
  }

  public function submitForm(array &$form, FormStateInterface $form_state) {

    $cache_action = $form_state->getValue('cache_action');

    if ('clear_url' === $cache_action) {
      $url = $form_state->getValue('clear_url_cache');

      if (!empty($url)) {
        $directory = GTFS_SCHEDULE_CACHE_LOCATION;
        $full_path = $directory . '/' . md5($url) . '.serialized';
        if(!file_exists($full_path)) {
          \Drupal::messenger()->addWarning($this->t('URL %url not cached', ['%url' => $url]));
        } else {
          unlink($full_path);
          \Drupal::messenger()->addMessage($this->t('Deleted %url', ['%url' => $url]));
        }
      }
    } else if ($cache_action) {
      gtfs_schedule_modify_cache($cache_action);
    }
  }

}